/* ++++++++++
	ucbserial.c
	A simple user-space front-end which allocates a PTY
	And then talks to the kernel driver
	Written by Martijn Coenen <M.F.A.Coenen@student.tue.nl>
	(C) 2001,2002
	
	Portions of code (and the idea) taken from the PPPoE driver
	Licensed under the GPL
+++++ */


#include <stdlib.h>
#include <stdio.h>
#include <OS.h>
#include <unistd.h>
#include "ucbserial.h"


/* Global pty struct */

serialcb_t		serialcb;

static status_t alloc_pty (char *buffer, int *fd);
static int32 watch_port(void *cookie);
static int32 block_read();

static status_t alloc_pty (char *buffer, int *fd) {
	char *letters = "pqrs";
	char *numbers = "0123456789abcdef";
	char *i, *j;
	int handle;
	
	for (i = letters; i[0]; i++) {
		for (j = numbers; j[0]; j++) {
			sprintf(buffer,"/dev/pt/%c%c",*i,*j);
			handle = open(buffer, O_RDWR, 0000);
			if (handle > 0) {
				*fd = handle;
				return B_OK;
			}
		}
	}
	
	return B_ERROR;
}


int main() {
	char		devname[64];
	thread_id	watcher;
	
	alloc_pty(serialcb.pty_name, &(serialcb.pty));
	serialcb.pty_name[5] = 't';
	sprintf(devname,"/dev/ports/cbserial");
	unlink(devname);
	symlink(serialcb.pty_name,devname);
	
	/* Non blocking? */
	serialcb.kernelfd = open("/dev/ports/kserial", O_RDWR);
	if (serialcb.kernelfd <= 0) {
		printf("Failed to open the kernel driver, exiting.\n");
	}
	printf("Linked %s to %s\n", devname, serialcb.pty_name);
	
	printf("Cardbus Serial Modem Driver starting...\n");
	printf("(C) Martijn Coenen 2001,2002\n");
	/* Now, spawn a watcher tread */
	watcher = spawn_thread(&watch_port,devname,10,NULL);

	resume_thread(watcher);
	
	block_read();
		
	return(0);

}

static int32 block_read() {
	char buffer[8192];
	ssize_t bufferSize;
	int i;
	
	while (true) {
		i = 0;
		bufferSize = read(serialcb.kernelfd,buffer,8192);
		if (bufferSize > 0)
			write(serialcb.pty, buffer, bufferSize);
	}
	return(0);
}

int32 watch_port(void *cookie) {
	
	ssize_t bufferSize;
	char buffer[8192];
	char devname[64];
	int i;
	sprintf(devname,"/dev/ports/cbserial");
		
	while (true) {
		i = 0;
		bufferSize = read(serialcb.pty,buffer,8192);
		if (bufferSize < 0) {
			alloc_pty(serialcb.pty_name,&(serialcb.pty));
			serialcb.pty_name[5] = 't';
			unlink(devname);
			symlink(serialcb.pty_name,devname);
			continue;
		}
		
		if (bufferSize > 0)
			write(serialcb.kernelfd, buffer, bufferSize);
	}
	return 0;
}


